from __future__ import annotations

from importlib import import_module
from pathlib import Path
from typing import Any, IO, TextIO

from ogc.na.util import deep_update


def apply_input_filter(stream: IO | bytes | str | Path, filters: dict[str, dict]) -> dict[str, Any] | list:
    filter_name, filter_conf = filters.popitem()

    metadata = {
        'filter': {
            'name': filter_name,
            'conf': filter_conf,
        },
    }

    try:
        filter_mod = import_module(f"ogc.na.input_filters.{filter_name}")
    except ImportError:
        raise ValueError(f'Cannot find input filter with name "{filter_name}"')

    if isinstance(stream, Path) or isinstance(stream, str):
        with open(stream, 'rb') as f:
            content = f.read()
        metadata['filename'] = str(stream)
    elif isinstance(stream, TextIO):
        content = stream.read().encode('utf-8')
    else:
        content = stream.read()

    data, filter_metadata = filter_mod.apply_filter(content, filter_conf)
    if filter_metadata:
        deep_update(metadata, filter_metadata, replace=True)

    return {
        'metadata': metadata,
        'data': data,
    }



import unittest
from pathlib import Path

from rdflib import URIRef, Namespace

from ogc.na.profile import ProfileRegistry

THIS_DIR = Path(__file__).parent
DATA_DIR = THIS_DIR / 'data'

EX = Namespace('http://example.org/prof/')


class ProfileTest(unittest.TestCase):

    def test_empty_profile_registry(self):
        empty = DATA_DIR / 'empty.ttl'
        registry = ProfileRegistry(empty)
        self.assertFalse(registry.profiles)

    def test_build_cyclic_profile_chain(self):
        registry = ProfileRegistry(DATA_DIR / 'profile_tree_cyclic.ttl')
        with self.assertRaises(ValueError):
            registry.build_profile_chain((EX.a, EX.c), recursive=True)
        with self.assertRaises(ValueError):
            registry.build_profile_chain((EX.a, EX.c), recursive=False)
        chain = registry.build_profile_chain((EX.a, EX.c), sort=False)
        self.assertEqual(len(chain), 4)
        chain = registry.build_profile_chain((EX.a, EX.c), sort=False, recursive=False)
        self.assertEqual(len(chain), 2)
        chain = registry.build_profile_chain((EX.a, EX.p, EX.q, EX.d), sort=False, recursive=False)
        self.assertEqual(len(chain), 2)

    def test_build_profile_chain(self):
        registry = ProfileRegistry(DATA_DIR / 'profile_tree.ttl')

        seq = (EX.f, EX.a)
        chain = registry.build_profile_chain(seq)
        self.assertSequenceEqual(chain, seq)

        seq = (EX.a, EX.f)
        chain = registry.build_profile_chain(seq)
        self.assertSequenceEqual(chain, seq)

        seq = (EX.e, EX.d)
        chain = registry.build_profile_chain(seq, recursive=False)
        self.assertSequenceEqual(chain, seq[::-1])

        seq = (EX.e, EX.d)
        chain = registry.build_profile_chain(seq, recursive=False, sort=False)
        self.assertSequenceEqual(chain, seq)

        chain = registry.build_profile_chain((EX.e, EX.f))
        self.assertEqual(len(chain), 5)  # All but EX.c
        for a, b in (
                (EX.a, EX.b),
                (EX.b, EX.d),
                (EX.b, EX.e),
                (EX.d, EX.e),
        ):
            self.assertLess(chain.index(a), chain.index(b))



#!/usr/bin/env python3
"""
This module defines auxiliary classes to represent [pySHACL](https://github.com/RDFLib/pySHACL)
validation reports.
"""
from __future__ import annotations
from pathlib import Path
from typing import Union, Iterable
from urllib.parse import urlsplit
from urllib.request import urlopen

import jsonschema
import requests
from rdflib import URIRef, Graph

from ogc.na import util


class ValidationReport:
    """
    Validation report from a single validation result.

    Structures a pySHACL tuple report into `result`, `graph` and `text`
    fields.
    """

    def __init__(self, pyshacl_result: tuple,
                 used_resources: Iterable[Union[str, Path]] = None):
        """
        :param pyshacl_result: result from executing [pyshacl.validate]
        """
        self.result, self.graph, self.text = pyshacl_result
        self.used_resources: set[Union[str, Path]] = set(used_resources) if used_resources else set()


class ProfileValidationReport:
    """
    Validation report for a given [profile](https://www.w3.org/TR/dx-prof/).
    """

    def __init__(self, profile_uri: URIRef, profile_token: str, report: ValidationReport):
        """
        :param profile_uri: URI for the profile
        :param profile_token: Token for the profile
        :param report: [ValidationReport][ogc.na.validation.ValidationReport]
        """
        self.profile_uri = profile_uri
        self.profile_token = profile_token
        self.report = report

    @property
    def used_resources(self) -> set[Union[str, Path]]:
        return self.report.used_resources

    @used_resources.setter
    def used_resources(self, ur: set[Union[str, Path]]):
        self.used_resources = ur


class ProfilesValidationReport:
    """
    Class to aggregate several [ProfileValidationReport][ogc.na.validation.ProfileValidationReport]'s
    coming from different profiles.

    Results are exposed through the following fields:

    * `reports`: list of [validation reports][ogc.na.validation.ProfileValidationReport]
    * `result`: `True` if all validations passed, otherwise `False`
    * `graph`: union of all SHACL validation report [Graph][rdflib.Graph]s
    * `text`: full report text coming from all validation results (separated by profile)
    """

    def __init__(self, profile_reports: list[ProfileValidationReport] = None):
        """
        :param profile_reports: list of initial [validation reports][ogc.na.validation.ProfileValidationReport]
        """
        self.reports: list[ProfileValidationReport] = []
        self.result = True
        self.graph = Graph()
        self.text = ''
        if profile_reports:
            for profile_report in self.reports:
                self.add(profile_report)

    def add(self, profile_report: ProfileValidationReport):
        """
        Add a new [validation report][ogc.na.validation.ProfileValidationReport].

        :param profile_report:
        """
        self.reports.append(profile_report)
        self.result &= profile_report.report.result
        util.copy_triples(profile_report.report.graph, self.graph)
        if profile_report.report.text:
            if self.text:
                self.text += '\n'
            self.text += (f"=== {profile_report.profile_token} "
                          f"({profile_report.profile_uri}) ===\n"
                          f"{profile_report.report.text}")

    def __contains__(self, item) -> bool:
        return any(r.profile_uri == item for r in self.reports)

    @property
    def used_resources(self):
        return set(r for report in self.reports for r in report.used_resources)


class YamlSchemaRefResolver(jsonschema.validators.RefResolver):

    def resolve_remote(self, uri):
        scheme = urlsplit(uri).scheme

        if scheme in self.handlers:
            result = self.handlers[scheme](uri)
        elif scheme in ["http", "https"]:
            result = util.load_yaml(content=requests.get(uri).content)
        else:
            # Otherwise, pass off to urllib and assume utf-8
            with urlopen(uri) as url:
                result = util.load_yaml(content=url.read().decode("utf-8"))

        if self.cache_remote:
            self.store[uri] = result
        return result
